//
//  LJColumnChartView.m
//  LJChartViewDemo
//
//  Created by liujun on 2018/4/17.
//  Copyright © 2018年 liujun. All rights reserved.
//

#import "LJColumnChartView.h"

#define TextHeight 30

@interface LJColumnChartView () <CAAnimationDelegate>

@property (nonatomic, strong) NSMutableArray *columnValueLabels;

@end

@implementation LJColumnChartView

#pragma mark - 懒加载

- (NSMutableArray *)columnValueLabels {
    if (!_columnValueLabels) {
        _columnValueLabels = [[NSMutableArray alloc] init];
    }
    return _columnValueLabels;
}

#pragma mark - 初始化方法

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setDefaultParameter];
    }
    return self;
}

- (void)setDefaultParameter {
    self.backgroundColor = [UIColor whiteColor];
    self.columnWidth = 10;
    self.animateDuration = 0.5;
    self.columnColor = [UIColor grayColor];
    self.scaleFontSize = 12;
    self.scaleColor = [UIColor grayColor];
    self.valueFontSize = 12;
    self.valueColor = [UIColor grayColor];
    self.xAxisColor = [UIColor grayColor];
}

#pragma mark - drawRect

- (void)drawRect:(CGRect)rect {
    if (self.columnChartItems.count <= 0) {
        return;
    }
    
    // x轴的y值
    CGFloat xAxisY = self.bounds.size.height - TextHeight;
    
    // x轴的x值
    CGFloat xAxisX = 0;
    
    // x轴的高
    CGFloat xAxisHeight = 0.5;
    
    // x轴的宽
    CGFloat xAxisWidth = self.bounds.size.width;
    
    // y轴的高（40为柱形图上的文字和x轴的刻度的高度和）
    CGFloat yAxisHeight = self.bounds.size.height - TextHeight * 2;
    
    // 所有值
    NSMutableArray *values = [[NSMutableArray alloc] init];
    for (LJColumnChartItem *item in self.columnChartItems) {
        [values addObject:item.value];
    }
    CGFloat maxValue = [[values valueForKeyPath:@"@max.doubleValue"] doubleValue];
    
    // 高度系数
    CGFloat percent = yAxisHeight / maxValue;
    
    // x轴总间隙
    CGFloat xAxisTotalMargin = xAxisWidth - self.columnChartItems.count * self.columnWidth;
    
    // 每个柱形图的间隙
    CGFloat columnMargin = xAxisTotalMargin / self.columnChartItems.count;
    
    // 第一个柱形图的x坐标
    CGFloat firstColumnX = columnMargin * 0.5;
    for (NSInteger i = 0; i < self.columnChartItems.count; i++) {
        LJColumnChartItem *item = self.columnChartItems[i];
        
        // 柱形图的y值
        CGFloat columnY = xAxisY  - percent * [item.value doubleValue];
        
        // 画柱形
        CAShapeLayer *columnLayer = [self drawColumnWithItem:item columnMargin:columnMargin firstColumnX:firstColumnX columnY:columnY xAxisY:xAxisY];
        
        // 添加动画
        [columnLayer addAnimation:[self makeAnimation] forKey:@"animation"];
        
        // 画x轴刻度值Label
        [self drawXAxisScaleValueLabelWithItem:item columnMargin:columnMargin firstColumnX:firstColumnX xAxisY:xAxisY];
        
        // 画柱形值Label
        [self drawColumnValueLabelWithItem:item columnMargin:columnMargin firstColumnX:firstColumnX columnY:columnY];
    }
    
    // 画x轴
    [self drawXAxisWithRect:CGRectMake(xAxisX, xAxisY, xAxisWidth, xAxisHeight)];
}

#pragma mark - 内部方法

- (CAShapeLayer *)drawColumnWithItem:(LJColumnChartItem *)item columnMargin:(CGFloat)columnMargin firstColumnX:(CGFloat)firstColumnX columnY:(CGFloat)columnY xAxisY:(CGFloat)xAxisY {
    // 画柱形
    // UIBezierPath *bezierPath = [UIBezierPath bezierPathWithRect:CGRectMake(firstColumnX + (self.columnWidth + columnMargin) * i , columnY, self.columnWidth, xAxisY - columnY)];
    NSInteger index = [self.columnChartItems indexOfObject:item];
    UIBezierPath *bezierPath = [UIBezierPath bezierPath];
    [bezierPath moveToPoint:CGPointMake(firstColumnX + (self.columnWidth + columnMargin) * index + self.columnWidth / 2, columnY)];
    [bezierPath addLineToPoint:CGPointMake(firstColumnX + (self.columnWidth + columnMargin) * index + self.columnWidth / 2, xAxisY)];
    
    CAShapeLayer *columnLayer = [CAShapeLayer layer];
    columnLayer.strokeColor = self.columnColor.CGColor;
    columnLayer.fillColor = [UIColor clearColor].CGColor;
    columnLayer.lineWidth = self.columnWidth;
    // columnLayer.lineWidth = 0.5;
    columnLayer.path = bezierPath.CGPath;
    [self.layer addSublayer:columnLayer];
    return columnLayer;
}

- (CABasicAnimation *)makeAnimation {
    CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"strokeStart"];
    animation.delegate = self;
    animation.duration = self.animateDuration;
    animation.fromValue = @(1);
    animation.toValue = @(0);
    animation.fillMode = kCAFillModeForwards;
    return animation;
}

- (void)drawColumnValueLabelWithItem:(LJColumnChartItem *)item columnMargin:(CGFloat)columnMargin firstColumnX:(CGFloat)firstColumnX columnY:(CGFloat)columnY {
    UILabel *columnValueLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.columnWidth + columnMargin * 0.5 , TextHeight)];
    columnValueLabel.textAlignment = NSTextAlignmentCenter;
    columnValueLabel.font = [UIFont systemFontOfSize:self.valueFontSize];
    columnValueLabel.textColor = self.scaleColor;
    columnValueLabel.center = CGPointMake(firstColumnX + (self.columnWidth + columnMargin) * [self.columnChartItems indexOfObject:item] + self.columnWidth / 2, columnY - TextHeight * 0.5);
    columnValueLabel.text = [NSString stringWithFormat:@"%@", item.value];
    columnValueLabel.alpha = 0;
    [self addSubview:columnValueLabel];
    [self.columnValueLabels addObject:columnValueLabel];
}

- (void)drawXAxisScaleValueLabelWithItem:(LJColumnChartItem *)item columnMargin:(CGFloat)columnMargin firstColumnX:(CGFloat)firstColumnX xAxisY:(CGFloat)xAxisY {
    UILabel *scaleValueLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.columnWidth + columnMargin * 0.5 , TextHeight)];
    scaleValueLabel.textAlignment = NSTextAlignmentCenter;
    scaleValueLabel.font = [UIFont systemFontOfSize:self.scaleFontSize];
    scaleValueLabel.textColor = self.scaleColor;
    scaleValueLabel.center = CGPointMake(firstColumnX + (self.columnWidth + columnMargin) * [self.columnChartItems indexOfObject:item] + self.columnWidth / 2, xAxisY + TextHeight * 0.5);
    scaleValueLabel.text = item.scaleValue;
    [self addSubview:scaleValueLabel];
}

- (void)drawXAxisWithRect:(CGRect)rect {
    UIBezierPath *linePath = [UIBezierPath bezierPath];
    [linePath moveToPoint:rect.origin];
    [linePath addLineToPoint:CGPointMake(rect.size.width, rect.origin.y)];
    
    CAShapeLayer *lineLayer = [CAShapeLayer layer];
    lineLayer.lineWidth = rect.size.height;
    lineLayer.strokeColor = self.xAxisColor.CGColor;
    lineLayer.path = linePath.CGPath;
    lineLayer.fillColor = [UIColor clearColor].CGColor;
    [self.layer addSublayer:lineLayer];
}

#pragma mark - CAAnimationDelegate

- (void)animationDidStop:(CAAnimation *)anim finished:(BOOL)flag {
    [self.columnValueLabels enumerateObjectsUsingBlock:^(UILabel *_Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [UIView animateWithDuration:self.animateDuration animations:^{
            obj.alpha = 1;
        }];
    }];
}

@end


@implementation LJColumnChartItem

@end

