//
//  LJLineChartView.m
//  LJChartViewDemo
//
//  Created by liujun on 2018/4/18.
//  Copyright © 2018年 liujun. All rights reserved.
//

#import "LJLineChartView.h"

#define XAxisScaleWidth 60
#define YAxisScaleHeight 30
#define N 10000

@interface LJLineChartView () <CAAnimationDelegate>

@property (nonatomic, strong) NSMutableArray *maxArray;
@property (nonatomic, strong) NSMutableArray *minArray;
@property (nonatomic, strong) NSMutableArray *rightScaleArray;
@property (nonatomic, strong) NSMutableArray<UIColor *> *rightScaleColorArray;
@property (nonatomic, strong) NSMutableArray *leftScaleLabelArray;
@property (nonatomic, strong) NSMutableArray *rightScaleLabelArray;

@end

@implementation LJLineChartView

#pragma mark - 懒加载

- (NSMutableArray *)maxArray {
    if (!_maxArray) {
        _maxArray = [[NSMutableArray alloc] init];
    }
    return _maxArray;
}

- (NSMutableArray *)minArray {
    if (!_minArray) {
        _minArray = [[NSMutableArray alloc] init];
    }
    return _minArray;
}

- (NSMutableArray<UIColor *> *)rightScaleArray {
    if (!_rightScaleArray) {
        _rightScaleArray = [[NSMutableArray alloc] init];
    }
    return _rightScaleArray;
}

- (NSMutableArray *)rightScaleColorArray {
    if (!_rightScaleColorArray) {
        _rightScaleColorArray = [[NSMutableArray alloc] init];
    }
    return _rightScaleColorArray;
}

- (NSMutableArray *)leftScaleLabelArray {
    if (!_leftScaleLabelArray) {
        _leftScaleLabelArray = [[NSMutableArray alloc] init];
    }
    return _leftScaleLabelArray;
}

- (NSMutableArray *)rightScaleLabelArray {
    if (!_rightScaleLabelArray) {
        _rightScaleLabelArray = [[NSMutableArray alloc] init];
    }
    return _rightScaleLabelArray;
}

#pragma mark - 初始化方法

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setDefaultParameter];
    }
    return self;
}

- (void)setDefaultParameter {
    self.backgroundColor = [UIColor whiteColor];
    self.yAxisScaleLineCount = 4;
    self.showRightYAxisScale = NO;
    self.indexInDataArray = 0;
    self.leftYAxisScaleWidth = 60;
    self.rightYAxisScaleWidth = 35;
    self.leftYAxisScaleAlignment = NSTextAlignmentLeft;
    self.rightYAxisScaleAlignment = NSTextAlignmentRight;
    self.leftYAxisScaleFontSize = 10;
    self.rightYAxisScaleFontSize = 10;
    self.leftYAxisScaleColor = [UIColor grayColor];
    self.rightYAxisScaleColor = [UIColor grayColor];
    self.lineWidth = 2;
    self.xAxisLineHeight = 0.5;
    self.xAxisLineColor = [UIColor grayColor];
    self.xAxisScaleFontSize = 10;
    self.xAxisScaleColor = [UIColor grayColor];
    self.topMargin = 10;
    self.xAxisScaleHeight = 30;
    self.animateDuration = 0.5;
}

#pragma mark - 最大值/最小值格式处理

-(double)formatMaxValue:(double)value
{
    int nCount=0;
    double tmpValue=fabs(value);
    bool tempFlag = false;
    if(value<0) {
        tempFlag = true;
    }
    for(nCount=0; tmpValue>10; nCount++){
        tmpValue=tmpValue/10;
    }
    
    int n=value/pow(10, nCount-1);
    int n1=n/10;
    int n2=n%10;
    
    if(n2==0){
        return value;
    }
    else if(n2>5){
        if(!tempFlag) {
            n2=0;
            n1++;
            if(n1==10){
                n1=1;
                nCount++;
            }
        } else {
            n2 = -5;
        }
    }
    else{
        if(!tempFlag) {
            n2=5;
        } else {
            n2=0;
        }
    }
    
    double result1=n1*pow(10, nCount);
    double result2=n2*pow(10, nCount-1);
    
    return result1+result2;
}

-(double)formatMinValue:(double)value
{
    int nCount=0;
    double tmpValue=fabs(value);
    bool tempFlag = false;
    if(value<0) {
        tempFlag = true;
    }
    for(nCount=0; tmpValue>1; nCount++){
        tmpValue=tmpValue/10;
    }
    
    double result=0;
    int n=tmpValue*10;
    if (tempFlag) {
        if(n<5){
            result=-5*pow(10, nCount-1);
        }
        else{
            result=-1*pow(10, nCount);
        }
    } else {
        if(n<5){
            result=pow(10, nCount-1);
        }
        else{
            result=5*pow(10, nCount-1);
        }
    }
    
    return result;
}

#pragma mark - drawRect

- (void)drawRect:(CGRect)rect {
    if (self.xAxisScaleValues.count < 2 || self.dataArray.count <= 0) {
        return;
    }
    
    for (NSArray *array in self.dataArray) {
        if (array.count < 2) {
            return;
        }
    }
    
    CGFloat rightMaxValue = 0;
    CGFloat rightMinValue = 0;
    CGFloat rightYAxisScaleMargin = 0;
    
    CGFloat leftMaxValue = 0;
    CGFloat leftMinValue = 0;
    CGFloat leftYAxisScaleMargin = 0;
    
    CGFloat xAxisWidth = 0;
    if (self.showRightYAxisScale) { // 右侧有百分比刻度
        
        // 从self.dataArray中抽出右侧刻度线需要的线条数据
        xAxisWidth = self.bounds.size.width - (self.leftYAxisScaleWidth + self.rightYAxisScaleWidth);
        [self.rightScaleArray addObject:self.dataArray[self.indexInDataArray]];
        
        //从self.dataArray中剔除右侧刻度线需要的线条数据，重新赋值给self.dataArray，以区分左右刻度值
        NSMutableArray *tempArray = [NSMutableArray arrayWithArray:self.dataArray];
        [tempArray removeObjectAtIndex:self.indexInDataArray];
        self.dataArray = tempArray;
        
        // 右刻度处理
        rightMaxValue = [[self.rightScaleArray.firstObject valueForKeyPath:@"@max.floatValue"] doubleValue] * N;
        rightMinValue = [[self.rightScaleArray.firstObject valueForKeyPath:@"@min.floatValue"] doubleValue] * N;
        rightYAxisScaleMargin = ([self formatMaxValue:rightMaxValue] / N - [self formatMinValue:rightMinValue] / N) / (self.yAxisScaleLineCount - 1);
        
        // 颜色处理
        [self.rightScaleColorArray addObject:self.colors[self.indexInDataArray]];
        NSMutableArray *colorTempArray = [NSMutableArray arrayWithArray:self.colors];
        [colorTempArray removeObjectAtIndex:self.indexInDataArray];
        self.colors = colorTempArray;
        
    } else { // 右边无任何刻度
        xAxisWidth = self.bounds.size.width - self.leftYAxisScaleWidth;
    }
    
    // 左刻度处理
    for (NSMutableArray *array in self.dataArray) {
        NSNumber *max = [array valueForKeyPath:@"@max.floatValue"];
        [self.maxArray addObject:max];
        
        NSNumber *min = [array valueForKeyPath:@"@min.floatValue"];
        [self.minArray addObject:min];
    }
    
    leftMaxValue = [[self.maxArray valueForKeyPath:@"@max.floatValue"] doubleValue] * N;
    leftMinValue = [[self.minArray valueForKeyPath:@"@min.floatValue"] doubleValue] * N;
    leftYAxisScaleMargin = ([self formatMaxValue:leftMaxValue] / N - [self formatMinValue:leftMinValue] / N) / (self.yAxisScaleLineCount - 1);
    
    CGFloat xAxisHeight = self.xAxisLineHeight;
    CGFloat xAxisX = self.leftYAxisScaleWidth;
    
    // 最底部一根横线（从下往上画线）
    CGFloat xAxisY = self.bounds.size.height - self.xAxisScaleHeight + self.topMargin;
    
    // 横线区域的高度
    CGFloat lineAreaHeight = self.bounds.size.height - self.xAxisScaleHeight - self.topMargin;
    
    // 每条横线之间的垂直间距
    CGFloat verticalMargin = lineAreaHeight / (self.yAxisScaleLineCount - 1);
    
    // 绘制水平线及左右刻度Lalel
    [self drawHorizontalLineWithXAxisHeight:xAxisHeight xAxisWidth:xAxisWidth xAxisX:xAxisX xAxisY:xAxisY verticalMargin:verticalMargin leftMinValue:leftMinValue leftMaxValue:leftMaxValue leftYAxisScaleMargin:leftYAxisScaleMargin rightMinValue:rightMinValue rightMaxValue:rightMaxValue rightYAxisScaleMargin:rightYAxisScaleMargin];
    
    // 绘制折线
    [self drawLineWithXAxisWidth:xAxisWidth lineAreaHeight:lineAreaHeight rightMaxValue:rightMaxValue rightMinValue:rightMinValue xAxisX:xAxisX xAxisY:xAxisY leftMaxValue:leftMaxValue leftMinValue:leftMinValue];
    
    // 绘制x轴刻度Label
    [self drawXAxisScaleWithXAxisWidth:xAxisWidth xAxisX:xAxisX xAxisY:xAxisY];
}

#pragma mark - 内部方法

- (void)drawLineWithXAxisWidth:(CGFloat)xAxisWidth lineAreaHeight:(CGFloat)lineAreaHeight rightMaxValue:(CGFloat)rightMaxValue  rightMinValue:(CGFloat)rightMinValue xAxisX:(CGFloat)xAxisX xAxisY:(CGFloat)xAxisY leftMaxValue:(CGFloat)leftMaxValue leftMinValue:(CGFloat)leftMinValue {
    
    // x轴数值刻度间距
    CGFloat xAxisValueScaleMargin = xAxisWidth / (self.dataArray.firstObject.count - 1);
    if (self.showRightYAxisScale) { // 有右刻度
        NSArray *scaleArry = self.rightScaleArray.firstObject;
        CGFloat rightPercent = lineAreaHeight / ([self formatMaxValue:rightMaxValue] / N - [self formatMinValue:rightMinValue] / N);
        UIBezierPath *linePath = [UIBezierPath bezierPath];
        CGPoint startPoint = CGPointMake(xAxisX, xAxisY - rightPercent * ([scaleArry.firstObject doubleValue] - [self formatMinValue:rightMinValue] / N));
        [linePath moveToPoint:startPoint];
        for (NSInteger i = 1; i < scaleArry.count; i++) {
            CGPoint endPoint = CGPointMake(xAxisX + xAxisValueScaleMargin * i, xAxisY - rightPercent * ([scaleArry[i] doubleValue] - [self formatMinValue:rightMinValue] / N));
            CGPoint oldPoint = CGPointMake(xAxisX + xAxisValueScaleMargin * (i - 1), xAxisY - rightPercent * ([scaleArry[i - 1] doubleValue] - [self formatMinValue:rightMinValue] / N));
            [linePath addCurveToPoint:endPoint controlPoint1:CGPointMake((endPoint.x + oldPoint.x) * 0.5, oldPoint.y) controlPoint2:CGPointMake((endPoint.x + oldPoint.x) * 0.5, endPoint.y)];
        }
        CAShapeLayer *lineLayer = [CAShapeLayer layer];
        lineLayer.lineWidth = self.lineWidth;
        lineLayer.strokeColor = self.rightScaleColorArray.firstObject.CGColor;
        lineLayer.path = linePath.CGPath;
        lineLayer.fillColor = [UIColor clearColor].CGColor;
        [self.layer addSublayer:lineLayer];
        [lineLayer addAnimation:[self makeAnimation] forKey:@"animation"];
    }
    
    CGFloat leftPercent = lineAreaHeight / ([self formatMaxValue:leftMaxValue] / N - [self formatMinValue:leftMinValue] / N);
    
    // 绘制折线
    for (NSMutableArray *array in self.dataArray) {
        UIBezierPath *linePath = [UIBezierPath bezierPath];
        CGPoint startPoint = CGPointMake(xAxisX, xAxisY - leftPercent * ([array.firstObject doubleValue] - [self formatMinValue:leftMinValue] / N));
        [linePath moveToPoint:startPoint];
        for (NSInteger i = 1; i < array.count; i++) {
            CGPoint endPoint = CGPointMake(xAxisX + xAxisValueScaleMargin * i, xAxisY - leftPercent * ([array[i] doubleValue] - [self formatMinValue:leftMinValue] / N));
            CGPoint oldPoint = CGPointMake(xAxisX + xAxisValueScaleMargin * (i - 1), xAxisY - leftPercent * ([array[i - 1] doubleValue] - [self formatMinValue:leftMinValue] / N));
            [linePath addCurveToPoint:endPoint controlPoint1:CGPointMake((endPoint.x + oldPoint.x) * 0.5, oldPoint.y) controlPoint2:CGPointMake((endPoint.x + oldPoint.x) * 0.5, endPoint.y)];
        }
        CAShapeLayer *lineLayer = [CAShapeLayer layer];
        lineLayer.lineWidth = self.lineWidth;
        lineLayer.strokeColor = self.colors[[self.dataArray indexOfObject:array]].CGColor;
        lineLayer.path = linePath.CGPath;
        lineLayer.fillColor = [UIColor clearColor].CGColor;
        [self.layer addSublayer:lineLayer];
        [lineLayer addAnimation:[self makeAnimation] forKey:@"animation"];
    }
}

- (void)drawHorizontalLineWithXAxisHeight:(CGFloat)xAxisHeight xAxisWidth:(CGFloat)xAxisWidth xAxisX:(CGFloat)xAxisX xAxisY:(CGFloat)xAxisY verticalMargin:(CGFloat)verticalMargin leftMinValue:(CGFloat)leftMinValue leftMaxValue:(CGFloat)leftMaxValue leftYAxisScaleMargin:(CGFloat)leftYAxisScaleMargin rightMinValue:(CGFloat)rightMinValue rightMaxValue:(CGFloat)rightMaxValue rightYAxisScaleMargin:(CGFloat)rightYAxisScaleMargin {
    
    // 绘制横线
    for (NSInteger i = 0; i < self.yAxisScaleLineCount; i++) {
        UIBezierPath *line = [[UIBezierPath alloc] init];
        line.lineWidth = xAxisHeight;
        [line moveToPoint:CGPointMake(xAxisX, xAxisY - verticalMargin * i)];
        [line addLineToPoint:CGPointMake(xAxisX + xAxisWidth, xAxisY - verticalMargin * i)];
        [self.xAxisLineColor setStroke];
        line.lineCapStyle = kCGLineCapButt;
        [line stroke];
        
        // y轴左边刻度Label
        UILabel *leftYScaleValueLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.leftYAxisScaleWidth, YAxisScaleHeight)];
        leftYScaleValueLabel.center = CGPointMake(self.leftYAxisScaleWidth * 0.5, xAxisY - verticalMargin * i);
        if (i == 0) { // 最小值
            leftYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f", [self formatMinValue:leftMinValue] / N];
        } else if (i == self.yAxisScaleLineCount - 1) { // 最大值
            leftYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f", [self formatMaxValue:leftMaxValue] / N];
        } else { // 除最小值和最大值
            leftYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f", [self formatMaxValue:leftMaxValue] / N + i * leftYAxisScaleMargin];
        }
        leftYScaleValueLabel.textAlignment = self.leftYAxisScaleAlignment;
        leftYScaleValueLabel.font = [UIFont systemFontOfSize:self.leftYAxisScaleFontSize];
        leftYScaleValueLabel.textColor = self.leftYAxisScaleColor;
        leftYScaleValueLabel.alpha = 0;
        [self addSubview:leftYScaleValueLabel];
        [self.leftScaleLabelArray addObject:leftYScaleValueLabel];
        
        if (self.showRightYAxisScale) { // 右侧有百分比刻度
            UILabel *rightYScaleValueLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.rightYAxisScaleWidth, YAxisScaleHeight)];
            rightYScaleValueLabel.center = CGPointMake(xAxisX + xAxisWidth + self.rightYAxisScaleWidth * 0.5, xAxisY - verticalMargin * i);
            if (i == 0) { // 最小值
                rightYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f%%", [self formatMinValue:rightMinValue] / N * 100];
            } else if (i == self.yAxisScaleLineCount - 1) { // 最大值
                rightYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f%%", [self formatMaxValue:rightMaxValue] / N * 100];
            } else { // 除最小值和最大值
                rightYScaleValueLabel.text = [NSString stringWithFormat:@"%.2f%%", ([self formatMinValue:rightMinValue] / N + i * rightYAxisScaleMargin) * 100];
            }
            rightYScaleValueLabel.textAlignment = self.rightYAxisScaleAlignment;
            rightYScaleValueLabel.font = [UIFont systemFontOfSize:self.rightYAxisScaleFontSize];
            rightYScaleValueLabel.textColor = self.rightYAxisScaleColor;
            rightYScaleValueLabel.alpha = 0;
            [self addSubview:rightYScaleValueLabel];
            [self.rightScaleLabelArray addObject:rightYScaleValueLabel];
        }
    }
}

- (void)drawXAxisScaleWithXAxisWidth:(CGFloat)xAxisWidth xAxisX:(CGFloat)xAxisX xAxisY:(CGFloat)xAxisY {
    
    // x轴刻度间距
    CGFloat xAxisScaleMargin = xAxisWidth / (self.xAxisScaleValues.count - 1);
    for (NSInteger i = 0; i < self.xAxisScaleValues.count; i++) {
        UILabel *scaleLabel = nil;
        if (i == 0) { // 第一个刻度Label
            scaleLabel = [[UILabel alloc] initWithFrame:CGRectMake(xAxisX, xAxisY, XAxisScaleWidth, self.xAxisScaleHeight)];
        } else if (i == self.xAxisScaleValues.count - 1) { // 最后一个刻度Label
            scaleLabel = [[UILabel alloc] initWithFrame:CGRectMake(xAxisX + xAxisWidth - XAxisScaleWidth, xAxisY, XAxisScaleWidth, self.xAxisScaleHeight)];
            scaleLabel.textAlignment = NSTextAlignmentRight;
        } else { // 除第一个和最后一个
            scaleLabel = [[UILabel alloc] initWithFrame:CGRectMake(xAxisX, xAxisY, XAxisScaleWidth, self.xAxisScaleHeight)];
            scaleLabel.center = CGPointMake(xAxisX + xAxisScaleMargin, xAxisY + self.xAxisScaleHeight * 0.5);
            scaleLabel.textAlignment = NSTextAlignmentRight;
        }
        scaleLabel.text = self.xAxisScaleValues[i];
        scaleLabel.font = [UIFont systemFontOfSize:self.xAxisScaleFontSize];
        scaleLabel.textColor = self.xAxisScaleColor;
        [self addSubview:scaleLabel];
    }
}

- (CABasicAnimation *)makeAnimation {
    CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"strokeEnd"];
    animation.delegate = self;
    animation.duration = self.animateDuration;
    animation.fromValue = @(0);
    animation.toValue = @(1);
    animation.fillMode = kCAFillModeForwards;
    return animation;
}

#pragma mark - CAAnimationDelegate

- (void)animationDidStop:(CAAnimation *)anim finished:(BOOL)flag {
    [self.leftScaleLabelArray enumerateObjectsUsingBlock:^(UILabel *_Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [UIView animateWithDuration:self.animateDuration animations:^{
            obj.alpha = 1;
        }];
    }];
    
    [self.rightScaleLabelArray enumerateObjectsUsingBlock:^(UILabel *_Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [UIView animateWithDuration:self.animateDuration animations:^{
            obj.alpha = 1;
        }];
    }];
}

@end
